import 'package:json_annotation/json_annotation.dart';
import 'package:date_format/date_format.dart' as DateFormatter;
import 'package:adeptutils/src/wrapper/adept_date_time.dart';

/// Created by Luis on 03/08/19.

/// This wrapper class is necessary because java.time.ZonedDateTime has issues with
/// Gson serialization, which requires a custom TypeAdapter to serialize/deserialize it...
/// But more important is that ZoneDateTime parsing from String requires Android min API level 26
/// when this project has defined min API level 21.
/// 
/// Default date format example: 2001-07-04T12:08:56.867-07:00
/// Default formatted date format example: Wed, 4 Jul 2001 12:08:56 PM


class ZonedDateTime extends AdeptDateTime
{
  static const String FULL_ZONED_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX";
  static const String SIMPLE_ZONED_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ssXXX";

  static const List<String> THREE_LETTER_ISO_8601_TIME_ZONE_FORMAT = ['yyyy','-','mm','-','dd','T','HH',':','nn',':','ss','.','SSS']; ///here the ZoneOffset is managed manually to add the colon(:) between two digit hours and two digit minutes
  static const List<String> TWO_LETTER_ISO_8601_TIME_ZONE_FORMAT = ['yyyy','-','mm','-','dd','T','HH',':','nn',':','ss','.','SSS','z'];
  static const List<String> FORMATTED_DATE_FORMAT_LIST = ['D',', ', 'd', ' ', 'M', ' ' ,'yyyy', ' ','hh',':','nn',':','ss', ' ', "am"];
  static const List<String> FORMATTED_TIME_FORMAT_LIST = ['hh',':','nn',':','ss', ' ', "am"];
  @JsonKey(ignore: true) List<String> _dateFormatList = ['yyyy','-','mm','-','dd','T','HH',':','nn',':','ss','.','SSS'];
  @JsonKey(ignore: true) List<String> _formattedDateFormatList = FORMATTED_DATE_FORMAT_LIST;
  @JsonKey(ignore: true) List<String> _formattedTimeFormatList = FORMATTED_TIME_FORMAT_LIST;

  ZonedDateTime({
    String date,
    int dateTimeMillis,
    DateTime dateAsDateTime,
    String dateFormatStandard,
    String formattedDateTimeFormat,
    String formattedDateFormat,
    String formattedTimeFormat,
    List<String> dateFormatList,
    List<String> formattedDateFormatList,
    List<String> formattedTimeFormatList,
  }) : super(
    date: date,
    dateTimeMillis: dateTimeMillis,
    dateAsDateTime: dateAsDateTime,
    dateFormatStandard: dateFormatStandard,
    formattedDateTimeFormat: formattedDateTimeFormat,
    formattedDateFormat: formattedDateFormat,
    formattedTimeFormat: formattedTimeFormat,
  ) {
    _dateFormatList = dateFormatList ?? THREE_LETTER_ISO_8601_TIME_ZONE_FORMAT;
    _formattedDateFormatList = formattedDateFormatList ?? FORMATTED_DATE_FORMAT_LIST;
    _formattedTimeFormatList = formattedTimeFormatList ?? FORMATTED_TIME_FORMAT_LIST;
    if(date == null){
      if(dateAsDateTime != null) this.dateAsDateTime = dateAsDateTime;
      if(dateTimeMillis != null) this.dateTimeMillis = dateTimeMillis;
    }
  }

  @override
  set dateAsDateTime(DateTime value) {
    if(value != null) super.dateAsDateTime = value;
    if(dateAsDateTime != null){
      super.dateAsDateTimeLocal = DateTime.fromMillisecondsSinceEpoch(dateAsDateTime.millisecondsSinceEpoch);
      _dateFormatList ??= THREE_LETTER_ISO_8601_TIME_ZONE_FORMAT;
      date = DateFormatter.formatDate(dateAsDateTime, _dateFormatList);
      if(_dateFormatList == THREE_LETTER_ISO_8601_TIME_ZONE_FORMAT){
        String zoneOffset = DateFormatter.formatDate(dateAsDateTime, ['z']);
        if(zoneOffset != null && zoneOffset.length == 5)
          zoneOffset = zoneOffset.substring(0, 3) + ":" + zoneOffset.substring(3, zoneOffset.length);
        date += zoneOffset;
      }
    }
  }

  List<String> get dateFormatList => _dateFormatList ?? (_dateFormatList = THREE_LETTER_ISO_8601_TIME_ZONE_FORMAT);
  set dateFormatList(List<String> value) => _dateFormatList = value ?? _dateFormatList;

  List<String> get formattedDateFormatList => _formattedDateFormatList ?? (_formattedDateFormatList = FORMATTED_DATE_FORMAT_LIST);
  set formattedDateFormatList(List<String> value) => _formattedDateFormatList = value ?? FORMATTED_DATE_FORMAT_LIST;

  List<String> get formattedTimeFormatList => _formattedTimeFormatList ?? (_formattedTimeFormatList = FORMATTED_TIME_FORMAT_LIST);
  set formattedTimeFormatList(List<String> value) => _formattedTimeFormatList = value ?? FORMATTED_TIME_FORMAT_LIST;

  factory ZonedDateTime.fromJson(String json) => json != null ? ZonedDateTime(date: json) : null;
  factory ZonedDateTime.fromJsonMap(Map<String, dynamic> json) => json != null ? ZonedDateTime(date: json['date'] as String) : null;
}